﻿using System.Drawing;

namespace HistoryDLL
{
    enum SelectType
    {
        //按寬高 (可能變形)
        HW = 0x0001,
        //指定寬, 高按比例改變
        W = 0x0010,
        //指定高, 寬按比例改變
        H = 0x0100,
        //指定高寬進行裁減(不變形)
        Cut = 0x1000
    };

    public class CutThumbnail
    {
        public CutThumbnail(string imagepath, int cutWidth, int cutHeight, string imageFormat, string ThumbPath)
        {
            MakeThumbnail(imagepath, ThumbPath, cutWidth, cutHeight, SelectType.Cut, imageFormat);
        }

        private void MakeThumbnail(string originalImagePath, string thumbnailPath, int width, int height, SelectType mode, string imageFormat)
        {
            System.Drawing.Image originalImage = System.Drawing.Image.FromFile(originalImagePath);

            int towidth = (int)width;
            int toheight = (int)height;

            int x = 0;
            int y = 0;
            int ow = originalImage.Width;
            int oh = originalImage.Height;

            switch (mode)
            {
                case SelectType.HW://指定高宽缩放（可能变形） 
                    break;
                case SelectType.W://指定宽，高按比例 
                    toheight = originalImage.Height * width / originalImage.Width;
                    break;
                case SelectType.H://指定高，宽按比例 
                    towidth = originalImage.Width * height / originalImage.Height;
                    break;
                case SelectType.Cut://指定高宽裁减（不变形） 
                    if ((double)originalImage.Width / (double)originalImage.Height > (double)towidth / (double)toheight)
                    {
                        oh = originalImage.Height;
                        ow = originalImage.Height * towidth / toheight;
                        y = 0;
                        x = (originalImage.Width - ow) / 2;
                    }
                    else
                    {
                        ow = originalImage.Width;
                        oh = originalImage.Width * height / towidth;
                        x = 0;
                        y = (originalImage.Height - oh) / 2;
                    }
                    break;
                default:
                    break;
            }
            //新建一个bmp图片 
            System.Drawing.Image bitmap = new System.Drawing.Bitmap(towidth, toheight);
            //新建一个画板 
            Graphics g = System.Drawing.Graphics.FromImage(bitmap);
            //设置高质量插值法 
            g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.High;
            //设置高质量,低速度呈现平滑程度 
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            //清空画布并以透明背景色填充 
            g.Clear(System.Drawing.Color.Transparent);
            //在指定位置并且按指定大小绘制原图片的指定部分 
            g.DrawImage(originalImage, new System.Drawing.Rectangle(0, 0, towidth, toheight), new System.Drawing.Rectangle(x, y, ow, oh), GraphicsUnit.Pixel);
            try
            {
                System.Drawing.Imaging.ImageFormat imgtype = System.Drawing.Imaging.ImageFormat.Jpeg;

                if (imageFormat == "jpg")
                {
                    imgtype = System.Drawing.Imaging.ImageFormat.Jpeg;
                }
                //保存缩略图
                bitmap.Save(thumbnailPath, imgtype);
            }
            catch (System.Exception e)
            {
                throw e;
            }
            finally
            {
                originalImage.Dispose();
                bitmap.Dispose();
                g.Dispose();
            }
        }
    }
}
